@@ -25,6 +25,7 @@ from oslo_config.cfg import NoSuchOptError
 from oslo_log import log as logging
 from oslo_middleware import cors
 from osprofiler import opts as profiler
+from oslo_log import versionutils
 
 from trove.common.i18n import _
 from trove.version import version_info as version
@@ -111,6 +112,9 @@ common_opts = [
                help='Service type to use when searching catalog.'),
     cfg.StrOpt('glance_endpoint_type', default='publicURL',
                help='Service endpoint type to use when searching catalog.'),
+    cfg.StrOpt('taskmanager_manager',
+               default='trove.taskmanager.manager.Manager',
+               help='Driver for taskmanager'),
     cfg.StrOpt('trove_url', help='URL without the tenant segment.'),
     cfg.StrOpt('trove_service_type', default='database',
                help='Service type to use when searching catalog.'),
@@ -200,7 +204,7 @@ common_opts = [
     cfg.IntOpt('agent_heartbeat_time', default=10,
                help='Maximum time (in seconds) for the Guest Agent to reply '
                     'to a heartbeat request.'),
-    cfg.IntOpt('agent_heartbeat_expiry', default=60,
+    cfg.IntOpt('agent_heartbeat_expiry', default=90,
                help='Time (in seconds) after which a guest is considered '
                     'unreachable'),
     cfg.IntOpt('num_tries', default=3,
@@ -220,6 +224,9 @@ common_opts = [
                default=10,
                help='Default maximum number of instances per tenant.',
                deprecated_name='max_instances_per_user'),
+    cfg.IntOpt('max_ram_per_tenant',
+               default=-1,
+               help='Default maximum total amount of RAM in MB per tenant.'),
     cfg.IntOpt('max_accepted_volume_size', default=10,
                help='Default maximum volume size (in GB) for an instance.'),
     cfg.IntOpt('max_volumes_per_tenant', default=40,
@@ -325,13 +332,36 @@ common_opts = [
     cfg.StrOpt('backup_swift_container', default='database_backups',
                help='Swift container to put backups in.'),
     cfg.BoolOpt('backup_use_gzip_compression', default=True,
-                help='Compress backups using gzip.'),
-    cfg.BoolOpt('backup_use_openssl_encryption', default=True,
-                help='Encrypt backups using OpenSSL.'),
-    cfg.StrOpt('backup_aes_cbc_key', default='default_aes_cbc_key',
-               help='Default OpenSSL aes_cbc key.'),
-    cfg.BoolOpt('backup_use_snet', default=False,
-                help='Send backup files over snet.'),
+                help='Compress backups using gzip.',
+                deprecated_for_removal=True,
+                deprecated_since=versionutils.deprecated.VICTORIA,
+                deprecated_reason='Backup data compression is enabled by '
+                                  'default. This option is ignored.'),
+    cfg.BoolOpt(
+        'backup_use_openssl_encryption', default=True,
+        help='Encrypt backups using OpenSSL.',
+        deprecated_for_removal=True,
+        deprecated_since=versionutils.deprecated.VICTORIA,
+        deprecated_reason='Trove should not encrypt backup data on '
+                          'behalf of the user. This option is ignored.'
+    ),
+    cfg.StrOpt(
+        'backup_aes_cbc_key', default='',
+        help='Default OpenSSL aes_cbc key for decrypting backup data created '
+             'prior to Victoria.',
+        deprecated_for_removal=True,
+        deprecated_since=versionutils.deprecated.VICTORIA,
+        deprecated_reason='This option is only for backward compatibility. '
+                          'Backups created after Victoria are not encrypted '
+                          'any more.'
+    ),
+    cfg.BoolOpt(
+        'backup_use_snet', default=False,
+        help='Send backup files over snet.',
+        deprecated_for_removal=True,
+        deprecated_since=versionutils.deprecated.VICTORIA,
+        deprecated_reason='This option is not supported any more.'
+    ),
     cfg.IntOpt('backup_chunk_size', default=2 ** 16,
                help='Chunk size (in bytes) to stream to the Swift container. '
                'This should be in multiples of 128 bytes, since this is the '
@@ -483,7 +513,11 @@ common_opts = [
                      'management.'),
     cfg.BoolOpt(
         'online_volume_resize', default=True,
-        help='If online volume resize is supported.')
+        help='If online volume resize is supported.'),
+    cfg.BoolOpt(
+        'enable_volume_az', default=False,
+        help='If true create the volume in the same availability-zone as the '
+             'instance'),
 ]
 
 
@@ -616,8 +650,10 @@ mysql_opts = [
         help='Database docker image.'
     ),
     cfg.StrOpt(
-        'backup_docker_image', default='openstacktrove/db-backup-mysql:1.0.0',
-        help='The docker image used for backup and restore.'
+        'backup_docker_image', default='openstacktrove/db-backup-mysql:1.1.0',
+        help='The docker image used for backup and restore. For mysql, '
+             'the minor version is added to the image name as a suffix before '
+             'creating container, e.g. openstacktrove/db-backup-mysql5.7:1.0.0'
     ),
 ]
 
@@ -1055,13 +1091,18 @@ postgresql_group = cfg.OptGroup(
     'postgresql', title='PostgreSQL options',
     help="Oslo option group for the PostgreSQL datastore.")
 postgresql_opts = [
+    cfg.BoolOpt(
+        'enable_clean_wal_archives',
+        default=True,
+        help='Enable the periodic job to clean up WAL archive folder.'
+    ),
     cfg.StrOpt(
         'docker_image', default='postgres',
         help='Database docker image.'
     ),
     cfg.StrOpt(
         'backup_docker_image',
-        default='openstacktrove/db-backup-postgresql:1.0.0',
+        default='openstacktrove/db-backup-postgresql:1.1.2',
         help='The docker image used for backup and restore.'
     ),
     cfg.BoolOpt('icmp', default=False,
@@ -1095,7 +1136,11 @@ postgresql_opts = [
     cfg.StrOpt('wal_archive_location', default='/mnt/wal_archive',
                help="Filesystem path storing WAL archive files when "
                     "WAL-shipping based backups or replication "
-                    "is enabled."),
+                    "is enabled.",
+               deprecated_for_removal=True,
+               deprecated_reason='Option is not used any more, will be '
+                                 'removed in future release.'
+               ),
     cfg.BoolOpt('root_on_create', default=False,
                 help='Enable the automatic creation of the root user for the '
                 'service during instance-create. The generated password for '
@@ -1118,7 +1163,7 @@ postgresql_opts = [
                     "statement logging.",
                deprecated_for_removal=True,
                deprecated_reason='Will be replaced by configuration group '
-               'option: log_min_duration_statement'),
+                                 'option: log_min_duration_statement'),
     cfg.IntOpt('default_password_length', default=36,
                help='Character length of generated passwords.',
                deprecated_name='default_password_length',
@@ -1382,7 +1427,7 @@ mariadb_opts = [
     ),
     cfg.StrOpt(
         'backup_docker_image',
-        default='openstacktrove/db-backup-mariadb:1.0.0',
+        default='openstacktrove/db-backup-mariadb:1.1.0',
         help='The docker image used for backup and restore.'
     ),
 ]
@@ -1399,7 +1444,7 @@ rpcapi_cap_opts = [
         'taskmanager', default='latest',
         help='Set a version cap for messages sent to taskmanager services'),
     cfg.StrOpt(
-        'guestagent', default='latest',
+        'guestagent', default='xena',
         help='Set a version cap for messages sent to guestagent services'),
     cfg.StrOpt(
         'conductor', default='latest',
@@ -1467,6 +1512,24 @@ service_credentials_opts = [
                help="Keystone region name of the Trove service project."),
 ]
 
+guest_agent_group = cfg.OptGroup(
+    'guest_agent', title='Guest Agent options',
+    help="Config options used by guest agent.")
+guest_agent_opts = [
+    cfg.StrOpt(
+        'container_registry',
+        help='URL to the registry. E.g. https://index.docker.io/v1/'
+    ),
+    cfg.StrOpt(
+        'container_registry_username',
+        help='The registry username.'
+    ),
+    cfg.StrOpt(
+        'container_registry_password',
+        help='The plaintext registry password.'
+    ),
+]
+
 CONF = cfg.CONF
 
 CONF.register_opts(path_opts)
@@ -1489,6 +1552,7 @@ CONF.register_group(db2_group)
 CONF.register_group(mariadb_group)
 CONF.register_group(network_group)
 CONF.register_group(service_credentials_group)
+CONF.register_group(guest_agent_group)
 
 CONF.register_opts(mysql_opts, mysql_group)
 CONF.register_opts(percona_opts, percona_group)
@@ -1504,6 +1568,7 @@ CONF.register_opts(db2_opts, db2_group)
 CONF.register_opts(mariadb_opts, mariadb_group)
 CONF.register_opts(network_opts, network_group)
 CONF.register_opts(service_credentials_opts, service_credentials_group)
+CONF.register_opts(guest_agent_opts, guest_agent_group)
 
 CONF.register_opts(rpcapi_cap_opts, upgrade_levels)
 
@@ -1521,9 +1586,11 @@ def list_opts():
         (None, path_opts + versions_opts + common_opts),
         ('database', database_opts),
         (mysql_group, mysql_opts),
+        (postgresql_group, postgresql_opts),
         (mariadb_group, mariadb_opts),
         (network_group, network_opts),
         (service_credentials_group, service_credentials_opts),
+        (guest_agent_group, guest_agent_opts),
     ]
 
     return keystone_middleware_opts + keystone_loading_opts + trove_opts
